#region License

/*
 
Copyright (c) 2010-2014 Danko Kozar

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 
*/

#endregion License

using eDriven.Gui.Editor.Util;

namespace eDriven.Gui.Editor.Dialogs
{
    internal class CSharpSnippetGenerator : ISnippetGenerator
    {
        private readonly AddHandlerDataObject _data;

        public CSharpSnippetGenerator(AddHandlerDataObject data)
        {
            _data = data;
        }

        public string Generate()
        {
            string methods = string.Empty;
            if (_data.AddComponentInstantiatedHandler)
            {
                methods += CreateMethod("ComponentInstantiated", @"// fires after the GUI component is instantiated");
            }
            if (_data.AddInitializeComponentHandler)
            {
                methods += CreateMethod("InitializeComponent", @"// fires after the GUI component is initialized");
            }
            if (!string.IsNullOrEmpty(_data.MethodName))
            {
                methods += CreateEventHandler();
            }

            return
                string.Format(
                    @"{2}

using UnityEngine;
using eDriven.Core.Events;
using eDriven.Gui;
using eDriven.Gui.Components;
using eDriven.Gui.Designer;
//using Event=eDriven.Core.Events.Event;

public class {0} : MonoBehaviour {{
{1}
}}", _data.ClassName, methods, AutogeneratedHeader.Generate());
        }

        public string CreateMethod(string methodName, string comment)
        {
            var content = null != _data.Adapter ?
                string.Format(@"{0} {1} = ({0})component;", _data.Adapter.ComponentType, StringUtil.TypeNameToCamelCase(_data.Adapter.ComponentType.Name)) : string.Empty;

            return string.Format(
                @"
    {1}public void {0}(eDriven.Gui.Components.Component component)
    {{
        {2}
    }}
", methodName, string.Format(@"{0}
    ", comment), content);
        }

        public string CreateEventHandler()
        {
            //string cast = string.Empty;
            //if (_data.Cast)
            //{
            //    cast = string.Format(@"{0} e2 = ({0})e;", _data.EventAttribute.Type);
            //}

            string content = _data.Cast && null != _data.EventAttribute ?
                string.Format(@"{0} e2 = ({0})e;
", _data.EventAttribute.Type) : string.Empty;

            if (_data.OpenScript)
                content += PersistedDataProcessor.CursorMarker;

            string comment = string.Empty;
            var name = _data.GetEventName();
            if (EventDescriptions.Instance.ContainsKey(name))
                comment = string.Format(@"// {0}
    ", EventDescriptions.Instance[name]);

            return string.Format(
                @"
    {2}public void {0}(eDriven.Core.Events.Event e)
    {{
        {1}
    }}
", _data.MethodName, content, comment);
        }
    }
}