﻿namespace eDriven.Gui.Events
{
    public static class EventPriority
    {
        /**
	     *  The CursorManager has handlers for mouse events
	     *  which must be executed before other mouse event handlers,
	     *  so they have a high priority.
	     */
	    public const int CURSOR_MANAGEMENT = 200;
    	
	    /**
	     *  Autogenerated event handlers that evaluate data-binding expressions
	     *  need to be executed before any others, so they have a higher priority
	     *  than the default.
	     */
	    //public const int BINDING = 100;

	    /**
	     *  Event handlers on component instances are executed with the default
	     *  priority, <code>0</code>.
	     */
	    public const int DEFAULT = 0;

	    /**
	     *  Some components listen to events that they dispatch on themselves
	     *  and let other listeners call the <code>preventDefault()</code>
	     *  method to tell the component not to perform a default action.
	     *  Those components must listen with a lower priority than the default
	     *  priority, so that the other handlers are executed first and have
	     *  a chance to call <code>preventDefault()</code>.
	     */
	    public const int DEFAULT_HANDLER = -50;

	    /**
	     *  Autogenerated event handlers that trigger effects are executed
	     *  after other event handlers on component instances, so they have
	     *  a lower priority than the default.
	     */
	    public const int EFFECT = -100;
    }
}
